@@ -79,6 +79,14 @@ input mode under Windows. */
 #define fileno _fileno
 #endif
 
+/* A user sent this fix for Borland Builder 5 under Windows. */
+
+#ifdef __BORLANDC__
+#define _setmode(handle, mode) setmode(handle, mode)
+#endif
+
+/* Not Windows */
+
 #else
 #include <sys/time.h>          /* These two includes are needed */
 #include <sys/resource.h>      /* for setrlimit(). */
@@ -181,6 +189,7 @@ static int locale_set = 0;
 static int show_malloc;
 static int use_utf8;
 static size_t gotten_store;
+static const unsigned char *last_callout_mark = NULL;
 
 /* The buffers grow automatically if very long input lines are encountered. */
 
@@ -189,6 +198,38 @@ static uschar *buffer = NULL;
 static uschar *dbuffer = NULL;
 static uschar *pbuffer = NULL;
 
+/* Textual explanations for runtime error codes */
+
+static const char *errtexts[] = {
+  NULL,  /* 0 is no error */
+  NULL,  /* NOMATCH is handled specially */
+  "NULL argument passed",
+  "bad option value",
+  "magic number missing",
+  "unknown opcode - pattern overwritten?",
+  "no more memory",
+  NULL,  /* never returned by pcre_exec() or pcre_dfa_exec() */
+  "match limit exceeded",
+  "callout error code",
+  NULL,  /* BADUTF8 is handled specially */
+  "bad UTF-8 offset",
+  NULL,  /* PARTIAL is handled specially */
+  "not used - internal error",
+  "internal error - pattern overwritten?",
+  "bad count value",
+  "item unsupported for DFA matching",
+  "backreference condition or recursion test not supported for DFA matching",
+  "match limit not supported for DFA matching",
+  "workspace size exceeded in DFA matching",
+  "too much recursion for DFA matching",
+  "recursion limit exceeded",
+  "not used - internal error",
+  "invalid combination of newline options",
+  "bad offset value",
+  NULL,  /* SHORTUTF8 is handled specially */
+  "nested recursion at the same subject position"
+};
+
 
 /*************************************************
 *         Alternate character tables             *
@@ -516,6 +557,30 @@ static const unsigned char tables1[] = {
 
 
 
+
+#ifndef HAVE_STRERROR
+/*************************************************
+*     Provide strerror() for non-ANSI libraries  *
+*************************************************/
+
+/* Some old-fashioned systems still around (e.g. SunOS4) don't have strerror()
+in their libraries, but can provide the same facility by this simple
+alternative function. */
+
+extern int   sys_nerr;
+extern char *sys_errlist[];
+
+char *
+strerror(int n)
+{
+if (n < 0 || n >= sys_nerr) return "unknown error number";
+return sys_errlist[n];
+}
+#endif /* HAVE_STRERROR */
+
+
+
+
 /*************************************************
 *        Read or extend an input line            *
 *************************************************/
@@ -896,6 +961,13 @@ fprintf(outfile, "%.*s", (cb->next_item_length == 0)? 1 : cb->next_item_length,
 fprintf(outfile, "\n");
 first_callout = 0;
 
+if (cb->mark != last_callout_mark)
+  {
+  fprintf(outfile, "Latest Mark: %s\n",
+    (cb->mark == NULL)? "<unset>" : (char *)(cb->mark));
+  last_callout_mark = cb->mark;
+  }
+
 if (cb->callout_data != NULL)
   {
   int callout_data = *((int *)(cb->callout_data));
@@ -1123,7 +1195,7 @@ printf("  -p       use POSIX interface\n");
 #endif
 printf("  -q       quiet: do not output PCRE version number at start\n");
 printf("  -S <n>   set stack size to <n> megabytes\n");
-printf("  -s       output store (memory) used information\n"
+printf("  -s       force each pattern to be studied\n"
        "  -t       time compilation and execution\n");
 printf("  -t <n>   time compilation and execution, repeating <n> times\n");
 printf("  -tm      time execution (matching) only\n");
@@ -1151,6 +1223,7 @@ int timeit = 0;
 int timeitm = 0;
 int showinfo = 0;
 int showstore = 0;
+int force_study = 0;
 int quiet = 0;
 int size_offsets = 45;
 int size_offsets_max;
@@ -1199,8 +1272,8 @@ while (argc > 1 && argv[op][0] == '-')
   {
   unsigned char *endptr;
 
-  if (strcmp(argv[op], "-s") == 0 || strcmp(argv[op], "-m") == 0)
-    showstore = 1;
+  if (strcmp(argv[op], "-m") == 0) showstore = 1;
+  else if (strcmp(argv[op], "-s") == 0) force_study = 1;
   else if (strcmp(argv[op], "-q") == 0) quiet = 1;
   else if (strcmp(argv[op], "-b") == 0) debug = 1;
   else if (strcmp(argv[op], "-i") == 0) showinfo = 1;
@@ -1234,7 +1307,7 @@ while (argc > 1 && argv[op][0] == '-')
       ((stack_size = get_value((unsigned char *)argv[op+1], &endptr)),
         *endptr == 0))
     {
-#if defined(_WIN32) || defined(WIN32)
+#if defined(_WIN32) || defined(WIN32) || defined(__minix)
     printf("PCRE: -S not supported on this OS\n");
     exit(1);
 #else
@@ -1370,13 +1443,16 @@ while (!done)
   const unsigned char *tables = NULL;
   unsigned long int true_size, true_study_size = 0;
   size_t size, regex_gotten_store;
+  int do_allcaps = 0;
   int do_mark = 0;
   int do_study = 0;
+  int no_force_study = 0;
   int do_debug = debug;
   int do_G = 0;
   int do_g = 0;
   int do_showinfo = showinfo;
   int do_showrest = 0;
+  int do_showcaprest = 0;
   int do_flip = 0;
   int erroroffset, len, delimiter, poffset;
 
@@ -1438,7 +1514,7 @@ while (!done)
         }
       }
 
-    fprintf(outfile, "Compiled regex%s loaded from %s\n",
+    fprintf(outfile, "Compiled pattern%s loaded from %s\n",
       do_flip? " (byte-inverted)" : "", p);
 
     /* Need to know if UTF-8 for printing data strings */
@@ -1446,7 +1522,7 @@ while (!done)
     new_info(re, NULL, PCRE_INFO_OPTIONS, &get_options);
     use_utf8 = (get_options & PCRE_UTF8) != 0;
 
-    /* Now see if there is any following study data */
+    /* Now see if there is any following study data. */
 
     if (true_study_size != 0)
       {
@@ -1542,7 +1618,11 @@ while (!done)
       case 's': options |= PCRE_DOTALL; break;
       case 'x': options |= PCRE_EXTENDED; break;
 
-      case '+': do_showrest = 1; break;
+      case '+':
+      if (do_showrest) do_showcaprest = 1; else do_showrest = 1;
+      break;
+
+      case '=': do_allcaps = 1; break;
       case 'A': options |= PCRE_ANCHORED; break;
       case 'B': do_debug = 1; break;
       case 'C': options |= PCRE_AUTO_CALLOUT; break;
@@ -1560,10 +1640,18 @@ while (!done)
       case 'P': do_posix = 1; break;
 #endif
 
-      case 'S': do_study = 1; break;
+      case 'S':
+      if (do_study == 0) do_study = 1; else
+        {
+        do_study = 0;
+        no_force_study = 1;
+        }
+      break;
+
       case 'U': options |= PCRE_UNGREEDY; break;
       case 'W': options |= PCRE_UCP; break;
       case 'X': options |= PCRE_EXTRA; break;
+      case 'Y': options |= PCRE_NO_START_OPTIMISE; break;
       case 'Z': debug_lengths = 0; break;
       case '8': options |= PCRE_UTF8; use_utf8 = 1; break;
       case '?': options |= PCRE_NO_UTF8_CHECK; break;
@@ -1743,10 +1831,12 @@ while (!done)
     true_size = ((real_pcre *)re)->size;
     regex_gotten_store = gotten_store;
 
-    /* If /S was present, study the regexp to generate additional info to
-    help with the matching. */
+    /* If -s or /S was present, study the regex to generate additional info to
+    help with the matching, unless the pattern has the SS option, which
+    suppresses the effect of /S (used for a few test patterns where studying is
+    never sensible). */
 
-    if (do_study)
+    if (do_study || (force_study && !no_force_study))
       {
       if (timeit > 0)
         {
@@ -1900,7 +1990,7 @@ while (!done)
       if (do_flip) all_options = byteflip(all_options, sizeof(all_options));
 
       if (get_options == 0) fprintf(outfile, "No options\n");
-        else fprintf(outfile, "Options:%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s\n",
+        else fprintf(outfile, "Options:%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s\n",
           ((get_options & PCRE_ANCHORED) != 0)? " anchored" : "",
           ((get_options & PCRE_CASELESS) != 0)? " caseless" : "",
           ((get_options & PCRE_EXTENDED) != 0)? " extended" : "",
@@ -1916,6 +2006,7 @@ while (!done)
           ((get_options & PCRE_UTF8) != 0)? " utf8" : "",
           ((get_options & PCRE_UCP) != 0)? " ucp" : "",
           ((get_options & PCRE_NO_UTF8_CHECK) != 0)? " no_utf8_check" : "",
+          ((get_options & PCRE_NO_START_OPTIMIZE) != 0)? " no_start_optimize" : "",
           ((get_options & PCRE_DUPNAMES) != 0)? " dupnames" : "");
 
       if (jchanged) fprintf(outfile, "Duplicate name status changes\n");
@@ -1983,9 +2074,12 @@ while (!done)
       /* Don't output study size; at present it is in any case a fixed
       value, but it varies, depending on the computer architecture, and
       so messes up the test suite. (And with the /F option, it might be
-      flipped.) */
+      flipped.) If study was forced by an external -s, don't show this
+      information unless -i or -d was also present. This means that, except
+      when auto-callouts are involved, the output from runs with and without
+      -s should be identical. */
 
-      if (do_study)
+      if (do_study || (force_study && showinfo && !no_force_study))
         {
         if (extra == NULL)
           fprintf(outfile, "Study returned NULL\n");
@@ -2063,7 +2157,10 @@ while (!done)
           }
         else
           {
-          fprintf(outfile, "Compiled regex written to %s\n", to_file);
+          fprintf(outfile, "Compiled pattern written to %s\n", to_file);
+
+          /* If there is study data, write it. */
+
           if (extra != NULL)
             {
             if (fwrite(extra->study_data, 1, true_study_size, f) <
@@ -2073,7 +2170,6 @@ while (!done)
                 strerror(errno));
               }
             else fprintf(outfile, "Study data written to %s\n", to_file);
-
             }
           }
         fclose(f);
@@ -2108,6 +2204,7 @@ while (!done)
     int getlist = 0;
     int gmatched = 0;
     int start_offset = 0;
+    int start_offset_sign = 1;
     int g_notempty = 0;
     int use_dfa = 0;
 
@@ -2121,6 +2218,7 @@ while (!done)
 
     pcre_callout = callout;
     first_callout = 1;
+    last_callout_mark = NULL;
     callout_extra = 0;
     callout_count = 0;
     callout_fail_count = 999999;
@@ -2240,7 +2338,13 @@ while (!done)
         continue;
 
         case '>':
+        if (*p == '-')
+          {
+          start_offset_sign = -1;
+          p++;
+          }
         while(isdigit(*p)) start_offset = start_offset * 10 + *p++ - '0';
+        start_offset *= start_offset_sign;
         continue;
 
         case 'A':  /* Option setting */
@@ -2313,7 +2417,9 @@ while (!done)
 #endif
           use_dfa = 1;
         continue;
+#endif
 
+#if !defined NODFA
         case 'F':
         options |= PCRE_DFA_SHORTEST;
         continue;
@@ -2496,9 +2602,9 @@ while (!done)
             (void)pchars(dbuffer + pmatch[i].rm_so,
               pmatch[i].rm_eo - pmatch[i].rm_so, outfile);
             fprintf(outfile, "\n");
-            if (i == 0 && do_showrest)
+            if (do_showcaprest || (i == 0 && do_showrest))
               {
-              fprintf(outfile, " 0+ ");
+              fprintf(outfile, "%2d+ ", (int)i);
               (void)pchars(dbuffer + pmatch[i].rm_eo, len - pmatch[i].rm_eo,
                 outfile);
               fprintf(outfile, "\n");
@@ -2640,25 +2746,42 @@ while (!done)
             }
           }
 
+        /* do_allcaps requests showing of all captures in the pattern, to check
+        unset ones at the end. */
+
+        if (do_allcaps)
+          {
+          new_info(re, NULL, PCRE_INFO_CAPTURECOUNT, &count);
+          count++;   /* Allow for full match */
+          if (count * 2 > use_size_offsets) count = use_size_offsets/2;
+          }
+
+        /* Output the captured substrings */
+
         for (i = 0; i < count * 2; i += 2)
           {
           if (use_offsets[i] < 0)
+            {
+            if (use_offsets[i] != -1)
+              fprintf(outfile, "ERROR: bad negative value %d for offset %d\n",
+                use_offsets[i], i);
+            if (use_offsets[i+1] != -1)
+              fprintf(outfile, "ERROR: bad negative value %d for offset %d\n",
+                use_offsets[i+1], i+1);
             fprintf(outfile, "%2d: <unset>\n", i/2);
+            }
           else
             {
             fprintf(outfile, "%2d: ", i/2);
             (void)pchars(bptr + use_offsets[i],
               use_offsets[i+1] - use_offsets[i], outfile);
             fprintf(outfile, "\n");
-            if (i == 0)
+            if (do_showcaprest || (i == 0 && do_showrest))
               {
-              if (do_showrest)
-                {
-                fprintf(outfile, " 0+ ");
-                (void)pchars(bptr + use_offsets[i+1], len - use_offsets[i+1],
-                  outfile);
-                fprintf(outfile, "\n");
-                }
+              fprintf(outfile, "%2d+ ", i/2);
+              (void)pchars(bptr + use_offsets[i+1], len - use_offsets[i+1],
+                outfile);
+              fprintf(outfile, "\n");
               }
             }
           }
@@ -2765,11 +2888,13 @@ while (!done)
       to advance the start offset, and continue. We won't be at the end of the
       string - that was checked before setting g_notempty.
 
-      Complication arises in the case when the newline option is "any" or
-      "anycrlf". If the previous match was at the end of a line terminated by
-      CRLF, an advance of one character just passes the \r, whereas we should
-      prefer the longer newline sequence, as does the code in pcre_exec().
-      Fudge the offset value to achieve this.
+      Complication arises in the case when the newline convention is "any",
+      "crlf", or "anycrlf". If the previous match was at the end of a line
+      terminated by CRLF, an advance of one character just passes the \r,
+      whereas we should prefer the longer newline sequence, as does the code in
+      pcre_exec(). Fudge the offset value to achieve this. We check for a
+      newline setting in the pattern; if none was set, use pcre_config() to
+      find the default.
 
       Otherwise, in the case of UTF-8 matching, the advance must be one
       character, not one byte. */
@@ -2794,6 +2919,7 @@ while (!done)
                     (d == -1)? PCRE_NEWLINE_ANY : 0;
             }
           if (((obits & PCRE_NEWLINE_BITS) == PCRE_NEWLINE_ANY ||
+               (obits & PCRE_NEWLINE_BITS) == PCRE_NEWLINE_CRLF ||
                (obits & PCRE_NEWLINE_BITS) == PCRE_NEWLINE_ANYCRLF)
               &&
               start_offset < len - 1 &&
@@ -2804,25 +2930,42 @@ while (!done)
             {
             while (start_offset + onechar < len)
               {
-              int tb = bptr[start_offset+onechar];
-              if (tb <= 127) break;
-              tb &= 0xc0;
-              if (tb != 0 && tb != 0xc0) onechar++;
+              if ((bptr[start_offset+onechar] & 0xc0) != 0x80) break;
+              onechar++;
               }
             }
           use_offsets[1] = start_offset + onechar;
           }
         else
           {
-          if (count == PCRE_ERROR_NOMATCH)
+          switch(count)
             {
+            case PCRE_ERROR_NOMATCH:
             if (gmatched == 0)
               {
               if (markptr == NULL) fprintf(outfile, "No match\n");
                 else fprintf(outfile, "No match, mark = %s\n", markptr);
               }
+            break;
+
+            case PCRE_ERROR_BADUTF8:
+            case PCRE_ERROR_SHORTUTF8:
+            fprintf(outfile, "Error %d (%s UTF-8 string)", count,
+              (count == PCRE_ERROR_BADUTF8)? "bad" : "short");
+            if (use_size_offsets >= 2)
+              fprintf(outfile, " offset=%d reason=%d", use_offsets[0],
+                use_offsets[1]);
+            fprintf(outfile, "\n");
+            break;
+
+            default:
+            if (count < 0 && (-count) < sizeof(errtexts)/sizeof(const char *))
+              fprintf(outfile, "Error %d (%s)\n", count, errtexts[-count]);
+            else
+              fprintf(outfile, "Error %d (Unexpected value)\n", count);
+            break;
             }
-          else fprintf(outfile, "Error %d\n", count);
+
           break;  /* Out of the /g loop */
           }
         }
